@@ -90,7 +90,7 @@ static	int tglob();
 char	**ftpglob();
 char	*globerr;
 char	*home;
-extern	int errno;
+/*extern	int errno;*/
 static	char *strspl(), *strend();
 char	**copyblk();
 
@@ -107,6 +107,24 @@ static	int globbed;
 static	char *entp;
 static	char **sortbas;
 
+/* copy a list of strings into newly allocated memory */
+char **copylist(char **v) {
+	int i = 0, cnt;
+	char **p, **pp, **vv;
+	/* v points to array of char string pointers */
+	vv = v;
+	while(*vv++)i++;
+	if(NULL == (p = (char **)malloc(sizeof(char **) * (i + 1)))) {
+		globerr = "Out of memory for copylist ptr list";
+		return NULL;
+	}	
+	cnt = i; pp = p; vv = v;
+	for(i = 0; i < cnt; i++)
+		*pp++ = strdup(*vv++);
+	*pp = NULL;
+	return p;
+}
+
 #ifdef USE_GLOB
 int expand_tilde(char *instr, char *outstr) {
 	char *cs = instr;
@@ -142,24 +160,6 @@ int expand_tilde(char *instr, char *outstr) {
 	return 0;
 }
 
-/* copy a list of strings into newly allocated memory */
-char **copylist(char **v) {
-	int i = 0, cnt;
-	char **p, **pp, **vv;
-	// v points to array of char string pointers
-	vv = v;
-	while(*vv++)i++;
-	if(NULL == (p = (char **)malloc(sizeof(char **) * (i + 1)))) {
-		globerr = "Out of memory for copylist ptr list";
-		return NULL;
-	}	
-	cnt = i; pp = p; vv = v;
-	for(i = 0; i < cnt; i++)
-		*pp++ = strdup(*vv++);
-	*pp = NULL;
-	return p;
-}
-
 char **ftpglob(char *v) {
 	char **outp;
 	static glob_t globbuf;
@@ -216,7 +216,7 @@ ftpglob(v)
 	globerr = 0;
 	rscan(vv, tglob);
 	if (gflag == 0)
-		return (copyblk(vv));
+		return (copylist(vv));
 
 	gpath = agpath; gpathp = gpath; *gpathp = 0;
 	lastgpathp = &gpath[sizeof agpath - 2];
