@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2001  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2003  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -38,6 +38,7 @@
  * 3. Redistributions in source or binary form must retain an intact copy
  *    of this copyright notice and list of conditions.
  */
+#include "t_defines.h"
 #include "srp.h"
 #include "t_pwd.h"	/* for preparams */
 #include "t_sha.h"
@@ -57,7 +58,7 @@ struct client_meth_st {
 
 #define CLIENT_CTXP(srp)    ((struct client_meth_st *)(srp)->meth_data)
 
-static int
+static SRP_RESULT
 srp2945_client_init(SRP * srp)
 {
   srp->magic = SRP_MAGIC_CLIENT;
@@ -67,7 +68,7 @@ srp2945_client_init(SRP * srp)
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_finish(SRP * srp)
 {
   if(srp->meth_data) {
@@ -77,36 +78,16 @@ srp2945_client_finish(SRP * srp)
   return SRP_SUCCESS;
 }
 
-static int
-BigIntegerToCstr(BigInteger x, cstr * out)
-{
-  if(cstr_set_length(out, (BigIntegerBitLen(x) + 7) / 8) < 0)
-    return SRP_ERROR;
-  if(cstr_set_length(out, BigIntegerToBytes(x, out->data)) < 0)
-    return SRP_ERROR;
-  return SRP_SUCCESS;
-}
-
-static int
+static SRP_RESULT
 srp2945_client_params(SRP * srp, const unsigned char * modulus, int modlen,
 		      const unsigned char * generator, int genlen,
 		      const unsigned char * salt, int saltlen)
 {
-  int i, rc;
-  struct t_preconf * tpc;
-  BigInteger q, r;
+  int i;
   unsigned char buf1[SHA_DIGESTSIZE], buf2[SHA_DIGESTSIZE];
   SHA1_CTX ctxt;
 
-  if(modulus == NULL || generator == NULL || salt == NULL)
-    return SRP_ERROR;
-
-  /* Set fields in SRP context */
-  srp->modulus = BigIntegerFromBytes(modulus, modlen);
-  srp->generator = BigIntegerFromBytes(generator, genlen);
-  if(srp->salt == NULL)
-    srp->salt = cstr_new();
-  cstr_setn(srp->salt, salt, saltlen);
+  /* Fields set by SRP_set_params */
 
   /* Update hash state */
   SHA1Init(&ctxt);
@@ -133,41 +114,10 @@ srp2945_client_params(SRP * srp, const unsigned char * modulus, int modlen,
   /* hash: (H(N) xor H(g)) | H(U) | s */
   SHA1Update(&CLIENT_CTXP(srp)->hash, salt, saltlen);
 
-  /* Now attempt to validate parameters */
-  if(BigIntegerBitLen(srp->modulus) < SRP_MIN_BITS)
-    return SRP_ERROR;
-
-  for(i = 0; i < t_getprecount(); ++i) {
-    tpc = t_getpreparam(i);
-    if(tpc->modulus.len == modlen && tpc->generator.len == genlen &&
-       memcmp(tpc->modulus.data, modulus, modlen) == 0 &&
-       memcmp(tpc->generator.data, generator, genlen) == 0) {
-      return SRP_SUCCESS;		/* Match found, done */
-    }
-  }
-
-  /* Now do a safe-primality and generator test */
-  if(!t_isprime(srp->modulus))
-    return SRP_ERROR;
-
-  rc = SRP_ERROR;
-  /* q = (n-1)/2 */
-  q = BigIntegerFromInt(0);
-  BigIntegerSubInt(q, srp->modulus, 1);
-  BigIntegerDivInt(q, q, 2);
-  if(t_isprime(q)) {
-    r = BigIntegerFromInt(0);		/* Check g^q == -1 (mod n) */
-    BigIntegerModExp(r, srp->generator, q, srp->modulus);
-    BigIntegerAddInt(r, r, 1);
-    if(BigIntegerCmp(r, srp->modulus) == 0)
-      rc = SRP_SUCCESS;
-    BigIntegerFree(r);
-  }
-  BigIntegerFree(q);
-  return rc;
+  return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_auth(SRP * srp, const unsigned char * a, int alen)
 {
   /* On the client, the authenticator is the raw password-derived hash */
@@ -175,12 +125,12 @@ srp2945_client_auth(SRP * srp, const unsigned char * a, int alen)
 
   /* verifier = g^x mod N */
   srp->verifier = BigIntegerFromInt(0);
-  BigIntegerModExp(srp->verifier, srp->generator, srp->password, srp->modulus);
+  BigIntegerModExp(srp->verifier, srp->generator, srp->password, srp->modulus, srp->bctx, srp->accel);
 
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_passwd(SRP * srp, const unsigned char * p, int plen)
 {
   SHA1_CTX ctxt;
@@ -205,12 +155,11 @@ srp2945_client_passwd(SRP * srp, const unsigned char * p, int plen)
   return r;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_genpub(SRP * srp, cstr ** result)
 {
   cstr * astr;
-  unsigned char sbuf[SRP_SECRET_MAXBYTES];
-  int slen = (SRP_SECRET_BITS(BigIntegerBitLen(srp->modulus)) + 7) / 8;
+  int slen = (SRP_get_secret_bits(BigIntegerBitLen(srp->modulus)) + 7) / 8;
 
   if(result == NULL)
     astr = cstr_new();
@@ -220,13 +169,14 @@ srp2945_client_genpub(SRP * srp, cstr ** result)
     astr = *result;
   }
 
-  t_random(sbuf, slen);
-  srp->secret = BigIntegerFromBytes(sbuf, slen);
+  cstr_set_length(astr, BigIntegerByteLen(srp->modulus));
+  t_random(astr->data, slen);
+  srp->secret = BigIntegerFromBytes(astr->data, slen);
   /* Force g^a mod n to "wrap around" by adding log[2](n) to "a". */
   BigIntegerAddInt(srp->secret, srp->secret, BigIntegerBitLen(srp->modulus));
   /* A = g^a mod n */
   srp->pubkey = BigIntegerFromInt(0);
-  BigIntegerModExp(srp->pubkey, srp->generator, srp->secret, srp->modulus);
+  BigIntegerModExp(srp->pubkey, srp->generator, srp->secret, srp->modulus, srp->bctx, srp->accel);
   BigIntegerToCstr(srp->pubkey, astr);
 
   /* hash: (H(N) xor H(g)) | H(U) | s | A */
@@ -240,7 +190,7 @@ srp2945_client_genpub(SRP * srp, cstr ** result)
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_key(SRP * srp, cstr ** result,
 		   const unsigned char * pubkey, int pubkeylen)
 {
@@ -253,8 +203,7 @@ srp2945_client_key(SRP * srp, cstr ** result,
   SHA1Init(&ctxt);
   SHA1Update(&ctxt, pubkey, pubkeylen);
   SHA1Final(dig, &ctxt);
-  srp->u = BigIntegerFromInt((dig[0] << 24) | (dig[1] << 16) |
-			     (dig[2] << 8) | dig[3]);
+  srp->u = BigIntegerFromBytes(dig, 4);
   if(BigIntegerCmpInt(srp->u, 0) == 0)
     return SRP_ERROR;
 
@@ -274,12 +223,13 @@ srp2945_client_key(SRP * srp, cstr ** result,
 
   /* compute gb^(a + ux) (mod N) */
   e = BigIntegerFromInt(0);
-  BigIntegerMul(e, srp->password, srp->u);
+  BigIntegerMul(e, srp->password, srp->u, srp->bctx);
   BigIntegerAdd(e, e, srp->secret);	/* e = a + ux */
 
   srp->key = BigIntegerFromInt(0);
-  BigIntegerModExp(srp->key, gb, e, srp->modulus);
+  BigIntegerModExp(srp->key, gb, e, srp->modulus, srp->bctx, srp->accel);
   BigIntegerFree(e);
+  BigIntegerFree(gb);
 
   /* convert srp->key into a session key, update hash states */
   s = cstr_new();
@@ -303,7 +253,7 @@ srp2945_client_key(SRP * srp, cstr ** result,
   return SRP_SUCCESS;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_verify(SRP * srp, const unsigned char * proof, int prooflen)
 {
   unsigned char expected[SHA_DIGESTSIZE];
@@ -315,7 +265,7 @@ srp2945_client_verify(SRP * srp, const unsigned char * proof, int prooflen)
     return SRP_ERROR;
 }
 
-static int
+static SRP_RESULT
 srp2945_client_respond(SRP * srp, cstr ** proof)
 {
   if(proof == NULL)
