@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2003  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2005  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -65,7 +65,7 @@
 #include <openssl/opensslv.h>
 #include <openssl/rand.h>
 #elif defined(TOMCRYPT)
-#include "mycrypt.h"
+#include "tomcrypt.h"
 static prng_state g_rng;
 static unsigned char entropy[32];
 #elif defined(CRYPTOLIB)
@@ -282,6 +282,8 @@ t_initrand()
   t_mgf1(crpool, sizeof(crpool), (unsigned char *) &preseed, sizeof(preseed));
   seedDesRandom(crpool, sizeof(crpool));
   memset(crpool, 0, sizeof(crpool));
+#elif defined(GCRYPT)
+  gcry_random_add_bytes((unsigned char *)&preseed, sizeof(preseed), -1);
 #else
   SHA1Init(&ctxt);
   SHA1Update(&ctxt, (unsigned char *) &preseed, sizeof(preseed));
@@ -341,6 +343,8 @@ t_random(data, size)
   RAND_bytes(data, size);
 #elif defined(TOMCRYPT)
   yarrow_read(data, size, &g_rng);
+#elif defined(GCRYPT)
+  gcry_randomize(data, size, GCRY_STRONG_RANDOM);
 #elif defined(CRYPTOLIB)
   randomBytes(data, size, PSEUDO);
 #else
